import os
import unittest
from textwrap import dedent

import pyexcel as pe


class TestDateFormat:
    def test_reading_date_format(self):
        """
        date     time
        25/12/14 11:11:11
        25/12/14 12:12:12
        01/01/15 13:13:13
        0.0      0.0
        """
        import datetime

        r = pe.get_sheet(
            file_name=os.path.join("tests", "fixtures", "date_field.xlsx"),
            library="pyexcel-xlsxr",
        )
        assert isinstance(r[1, 0], datetime.date)
        assert r[1, 0].strftime("%d/%m/%y") == "25/12/14"
        assert isinstance(r[1, 1], datetime.time) is True
        assert r[1, 1].strftime("%H:%M:%S") == "11:11:11"
        value = r[4, 0].isoformat()
        assert value == "1899-12-30T00:00:00"
        assert r[4, 1].isoformat() == "00:00:00"


class TestAutoDetectInt(unittest.TestCase):
    def setUp(self):
        self.content = [[1, 2, 3.1]]
        self.test_file = "test_auto_detect_init.xlsx"
        pe.save_as(array=self.content, dest_file_name=self.test_file)

    def test_auto_detect_int(self):
        sheet = pe.get_sheet(file_name=self.test_file, library="pyexcel-xlsxr")
        expected = dedent(
            """
        pyexcel_sheet1:
        +---+---+-----+
        | 1 | 2 | 3.1 |
        +---+---+-----+"""
        ).strip()
        assert str(sheet) == expected

    def test_get_book_auto_detect_int(self):
        book = pe.get_book(file_name=self.test_file, library="pyexcel-xlsxr")
        expected = dedent(
            """
        pyexcel_sheet1:
        +---+---+-----+
        | 1 | 2 | 3.1 |
        +---+---+-----+"""
        ).strip()
        assert str(book) == expected

    def test_auto_detect_int_false(self):
        sheet = pe.get_sheet(
            file_name=self.test_file,
            auto_detect_int=False,
            library="pyexcel-xlsxr",
        )
        expected = dedent(
            """
        pyexcel_sheet1:
        +-----+-----+-----+
        | 1.0 | 2.0 | 3.1 |
        +-----+-----+-----+"""
        ).strip()
        assert str(sheet) == expected

    def test_get_book_auto_detect_int_false(self):
        book = pe.get_book(
            file_name=self.test_file,
            auto_detect_int=False,
            library="pyexcel-xlsxr",
        )
        expected = dedent(
            """
        pyexcel_sheet1:
        +-----+-----+-----+
        | 1.0 | 2.0 | 3.1 |
        +-----+-----+-----+"""
        ).strip()
        assert str(book) == expected

    def tearDown(self):
        os.unlink(self.test_file)



import os

import pyexcel
from base import create_sample_file1


class TestStringIO:
    def test_xlsx_stringio(self):
        testfile = "cute.xlsx"
        create_sample_file1(testfile)
        with open(testfile, "rb") as f:
            content = f.read()
            r = pyexcel.get_sheet(
                file_type="xlsx", file_content=content, library="pyexcel-xlsxr"
            )
            result = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", 1.1, 1]
            actual = list(r.enumerate())
            assert result == actual
        if os.path.exists(testfile):
            os.unlink(testfile)

    def test_xlsx_output_stringio(self):
        data = [[1, 2, 3], [4, 5, 6]]
        io = pyexcel.save_as(dest_file_type="xlsx", array=data)
        r = pyexcel.get_sheet(
            file_type="xlsx",
            file_content=io.getvalue(),
            library="pyexcel-xlsxr",
        )
        result = [1, 2, 3, 4, 5, 6]
        actual = list(r.enumerate())
        assert result == actual



from datetime import time, datetime

from pyexcel_xlsxr.messy_xlsx import (
    parse_row,
    find_sheets,
    parse_styles,
    get_sheet_index,
    column_to_number,
    parse_xfs_styles,
    parse_shared_strings,
    parse_book_properties,
)


def test_get_sheet_index():
    samples = [
        "xl/worksheets/sheet1.xml",
        "xl/worksheets/worksheet2.xml",
        "xl/worksheets/sheet.xml",
        "xl/worksheets/worksheet.xml",
    ]
    expected = [0, 1, 0, 0]
    actual = [get_sheet_index(file_name) for file_name in samples]
    assert actual == expected


def test_list_one():
    test_sample = [
        "_rels/.rels",
        "xl/_rels/workbook.xml.rels",
        "xl/worksheets/sheet2.xml",
        "xl/worksheets/sheet3.xml",
        "xl/worksheets/sheet1.xml",
        "xl/sharedStrings.xml",
        "xl/workbook.xml",
        "xl/styles.xml",
        "docProps/app.xml",
        "docProps/core.xml",
        "[Content_Types].xml",
    ]

    expected = [
        "xl/worksheets/sheet2.xml",
        "xl/worksheets/sheet3.xml",
        "xl/worksheets/sheet1.xml",
    ]

    sheet_files = find_sheets(test_sample)
    assert sheet_files == expected


def test_alternative_file_list():
    test_sample = [
        "_rels/.rels",
        "xl/_rels/workbook.xml.rels",
        "xl/worksheets/worksheet2.xml",
        "xl/worksheets/worksheet3.xml",
        "xl/worksheets/worksheet1.xml",
        "xl/sharedStrings.xml",
        "xl/workbook.xml",
        "xl/styles.xml",
        "docProps/app.xml",
        "docProps/core.xml",
        "[Content_Types].xml",
    ]

    expected = [
        "xl/worksheets/worksheet2.xml",
        "xl/worksheets/worksheet3.xml",
        "xl/worksheets/worksheet1.xml",
    ]

    sheet_files = find_sheets(test_sample)
    assert sheet_files == expected


def test_single_sheet():
    test_sample = [
        "_rels/.rels",
        "xl/_rels/workbook.xml.rels",
        "xl/worksheets/sheet.xml",
        "xl/sharedStrings.xml",
        "xl/workbook.xml",
        "xl/styles.xml",
        "docProps/app.xml",
        "docProps/core.xml",
        "[Content_Types].xml",
    ]

    expected = ["xl/worksheets/sheet.xml"]

    sheet_files = find_sheets(test_sample)
    assert sheet_files == expected


def test_alternative_single_sheet():
    test_sample = [
        "_rels/.rels",
        "xl/_rels/workbook.xml.rels",
        "xl/worksheets/worksheet.xml",
        "xl/sharedStrings.xml",
        "xl/workbook.xml",
        "xl/styles.xml",
        "docProps/app.xml",
        "docProps/core.xml",
        "[Content_Types].xml",
    ]

    expected = ["xl/worksheets/worksheet.xml"]

    sheet_files = find_sheets(test_sample)
    assert sheet_files == expected


def test_parse_row():
    xml_string = b"""
       <row collapsed="false" customFormat="false"
         customHeight="false" hidden="false" ht="12.75" outlineLevel="0" r="4">
          <c r="A4" s="1" t="n"><v>42005</v></c><c r="B4" s="2" t="n">
          <v>0.550844907407407</v>
          </c>
        </row>""".replace(
        b"\n", b" "
    )

    class Book:
        def __init__(self):
            self.xfs_styles = [1, 1, 2]
            self.styles = {"1": "dd/mm/yy", "2": "h:mm:ss;@"}
            self.properties = {"date1904": False}

    data = parse_row(xml_string, Book())
    assert [cell for cell in data] == [
        datetime(year=2015, month=1, day=1),
        time(hour=13, minute=13, second=13),
    ]


def test_parse_styles():
    sample = b"""
     <styleSheet
       xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
     <numFmts count="3"><numFmt formatCode="GENERAL" numFmtId="164"/>
     <numFmt formatCode="DD/MM/YY" numFmtId="165"/>
     <numFmt formatCode="H:MM:SS;@" numFmtId="166"/>
     </numFmts><fonts count="4"><font>
    <name val="Arial"/>""".replace(
        b"\n", b" "
    )
    styles = parse_styles(sample)
    assert list(styles.values()) == ["general", "dd/mm/yy", "h:mm:ss;@"]


def test_parse_properties():
    sample = b"""
     <workbook
       xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main"
    xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
         <fileVersion appName="Calc"/>
         <workbookPr backupFile="false" showObjects="all" date1904="false"/>
    <workbookProtection/>""".replace(
        b"\n", b" "
    )
    properties = parse_book_properties(sample)
    assert properties == {"date1904": False, "sheets": []}


def test_parse_sheet_properties():
    sample = b"""
     </bookViews><sheets>
        <sheet name="Sheet1" sheetId="1" state="visible" r:id="rId2"/>
        <sheet name="Sheet2" sheetId="2" state="visible" r:id="rId3"/>
        <sheet name="Sheet3" sheetId="3" state="visible" r:id="rId4"/>
        </sheets>
    <calcPr iterateCount="100" refMode="A1"
    iterate="false" iterateDelta="0.001"/>""".replace(
        b"\n", b" "
    )
    properties = parse_book_properties(sample)
    assert properties == {"sheets": ["Sheet1", "Sheet2", "Sheet3"]}


def test_parse_xfs_styles():
    sample = b"""
    <cellXfs count="3">
      <xf applyAlignment="false"
          applyBorder="false" applyFont="false"
          applyProtection="false"
          borderId="0" fillId="0" fontId="0" numFmtId="164" xfId="0">
      </xf>
     <xf applyAlignment="false"
         applyBorder="false" applyFont="false"
         applyProtection="false"
         borderId="0" fillId="0" fontId="0" numFmtId="165" xfId="0">
     </xf>
     <xf applyAlignment="false"
         applyBorder="false" applyFont="false"
         applyProtection="false"
         borderId="0" fillId="0" fontId="0" numFmtId="166" xfId="0">
     </xf>
    </cellXfs><cellStyles count="6">""".replace(
        b"\n", b" "
    )
    xfs_styles = parse_xfs_styles(sample)
    assert xfs_styles == [164, 165, 166]


def test_parse_shared_strings():
    sample = b"""
    <sst count="2" uniqueCount="2"
       xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
    <si><t>Date</t></si><si><t>Time</t></si></sst>""".replace(
        b"\n", b" "
    )
    content = parse_shared_strings(sample)
    assert list(content) == ["Date", "Time"]


def test_column_to_number_a1():
    assert column_to_number("A1") == 1


def test_column_to_number_z1():
    assert column_to_number("Z1") == 26


def test_column_to_number_aa2():
    assert column_to_number("AA2") == 27


def test_column_to_number_lowercase_az1():
    assert column_to_number("az1") == 52


def test_column_to_number_lowercase_ba1():
    assert column_to_number("ba1") == 53


def test_column_to_number_aaa():
    assert column_to_number("AAA1") == 703


def test_column_to_number_xfd():
    assert column_to_number("XFD1") == 16384



import io
import re
import zipfile
from datetime import time, datetime, timedelta
from functools import cache

from lxml import etree
from pyexcel_io._compact import OrderedDict

STYLE_FILENAME = "xl/styles.xml"
SHARED_STRING = "xl/sharedStrings.xml"
WORK_BOOK = "xl/workbook.xml"
SHEET_MATCHER = re.compile(r"xl/worksheets/(?:work)?sheet([0-9]+)?.xml")
XLSX_ROW_MATCH = re.compile(rb"<row\b[^>]*>.*?</row>", re.DOTALL)
NUMBER_FMT_MATCHER = re.compile(rb"<numFmts\b[^>]*>.*?</numFmts>", re.DOTALL)
XFS_FMT_MATCHER = re.compile(rb"<cellXfs\b[^>]*>.*?</cellXfs>", re.DOTALL)
SHEET_FMT_MATCHER = re.compile(rb"<sheet\b.*?/>", re.DOTALL)
DATE_1904_MATCHER = re.compile(rb"<workbookPr\b.*?/>", re.DOTALL)
# "xmlns:x14ac="http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac"
# But it not used for now
X14AC_NAMESPACE = b'xmlns:x14ac="http://not.used.com/"'

# see also ruby-roo lib at: http://github.com/hmcgowan/roo
FORMATS = {
    "general": "float",
    "0": "float",
    "0.00": "float",
    "#,##0": "float",
    "#,##0.00": "float",
    "0%": "percentage",
    "0.00%": "percentage",
    "0.00e+00": "float",
    "mm-dd-yy": "date",
    "d-mmm-yy": "date",
    "d-mmm": "date",
    "mmm-yy": "date",
    "h:mm am/pm": "date",
    "h:mm:ss am/pm": "date",
    "h:mm": "time",
    "h:mm:ss": "time",
    "m/d/yy h:mm": "date",
    "#,##0 ;(#,##0)": "float",
    "#,##0 ;[red](#,##0)": "float",
    "#,##0.00;(#,##0.00)": "float",
    "#,##0.00;[red](#,##0.00)": "float",
    "mm:ss": "time",
    "[h]:mm:ss": "time",
    "mmss.0": "time",
    "##0.0e+0": "float",
    "@": "float",
    "yyyy\\-mm\\-dd": "date",
    "dd/mm/yy": "date",
    "hh:mm:ss": "time",
    "dd/mm/yy\\ hh:mm": "date",
    "dd/mm/yyyy hh:mm:ss": "date",
    "yy-mm-dd": "date",
    "d-mmm-yyyy": "date",
    "m/d/yy": "date",
    "m/d/yyyy": "date",
    "dd-mmm-yyyy": "date",
    "dd/mm/yyyy": "date",
    "mm/dd/yy h:mm am/pm": "date",
    "mm/dd/yy hh:mm": "date",
    "mm/dd/yyyy h:mm am/pm": "date",
    "mm/dd/yyyy hh:mm:ss": "date",
    "yyyy-mm-dd hh:mm:ss": "date",
    "#,##0;(#,##0)": "float",
    '_(* #,##0_);_(* (#,##0);_(* "-"??_);_(@_)': "float",
    '_(* #,##0.00_);_(* (#,##0.00);_(* "-"??_);_(@_)': "float",
}
STANDARD_FORMATS = {
    0: "general",
    1: "0",
    2: "0.00",
    3: "#,##0",
    4: "#,##0.00",
    9: "0%",
    10: "0.00%",
    11: "0.00e+00",
    12: "# ?/?",
    13: "# ??/??",
    14: "mm-dd-yy",
    15: "d-mmm-yy",
    16: "d-mmm",
    17: "mmm-yy",
    18: "h:mm am/pm",
    19: "h:mm:ss am/pm",
    20: "h:mm",
    21: "h:mm:ss",
    22: "m/d/yy h:mm",
    37: "#,##0 ;(#,##0)",
    38: "#,##0 ;[red](#,##0)",
    39: "#,##0.00;(#,##0.00)",
    40: "#,##0.00;[red](#,##0.00)",
    45: "mm:ss",
    46: "[h]:mm:ss",
    47: "mmss.0",
    48: "##0.0e+0",
    49: "@",
}


class XLSXTable(object):
    def __init__(self, name, file_content, book):
        self.name = name
        self.content = file_content
        self.book = book

    def raw(self):
        rows = XLSX_ROW_MATCH.findall(self.content)
        for row in rows:
            yield parse_row(row, self.book)


class XLSXBookSet(object):
    def __init__(self, file_alike, **keywords):
        if hasattr(file_alike, "read"):
            file_alike = io.BytesIO(file_alike.read())
        self.zip_file = zipfile.ZipFile(file_alike)
        self.styles, self.xfs_styles = self.__extract_styles()
        self.properties = self.__extract_book_properties()
        self.shared_strings = list(self.__extract_shared_strings())

    def __extract_shared_strings(self):
        try:
            shared_string_content = self.zip_file.open(SHARED_STRING).read()
            return parse_shared_strings(shared_string_content)
        except KeyError:
            return []

    def __extract_styles(self):
        style_content = self.zip_file.open(STYLE_FILENAME).read()
        return parse_styles(style_content), parse_xfs_styles(style_content)

    def __extract_book_properties(self):
        book_content = self.zip_file.open(WORK_BOOK).read()
        return parse_book_properties(book_content)

    def close(self):
        if self.zip_file:
            self.zip_file.close()

    def make_tables(self):
        sheet_files = find_sheets(self.zip_file.namelist())
        for sheet_file in sorted(sheet_files):
            content = self.zip_file.open(sheet_file).read()
            sheet_index = get_sheet_index(sheet_file)
            sheet_name = self.properties["sheets"][sheet_index]
            yield XLSXTable(sheet_name, content, self)


def find_sheets(file_list):

    return [
        sheet_file
        for sheet_file in file_list
        if SHEET_MATCHER.match(sheet_file)
    ]


def get_sheet_index(file_name):
    sheet_match = SHEET_MATCHER.match(file_name)

    if sheet_match:
        index = int(sheet_match.group(1)) if sheet_match.group(1) else 1
        return index - 1
    else:
        raise Exception("Invalid sheet file name")


class Cell(object):
    def __init__(self):
        self.column_type = ""
        self.style_string = ""
        self.value = ""
        self.type = ""

    def __repr__(self):
        return str(self.value)


@cache
def column_to_number(column):
    column = re.sub(r"[^A-Z]", "", column.upper())

    result = 0

    for index, c in enumerate(column):
        result = result * 26 + (ord(c) - ord("A") + 1)

    return result


def parse_row(row_xml_string, book):
    if b"x14ac" in row_xml_string:
        row_xml_string = row_xml_string.replace(
            b"<row", (b"<row " + X14AC_NAMESPACE)
        )
    partial = io.BytesIO(row_xml_string)
    cells = []
    cell = Cell()

    last_column_number = None
    for action, element in etree.iterparse(partial):
        if element.tag in ["v", "t"]:
            cell.value = element.text
        elif element.tag in ["c"]:
            ref = element.attrib.get("r")
            if ref:
                column_number = column_to_number(ref)
                if last_column_number is not None:
                    padding = column_number - last_column_number - 1
                    if padding > 0:
                        cells += [Cell() for _ in range(padding)]
                last_column_number = column_number

            local_type = element.attrib.get("t")
            cell.column_type = local_type
            style_int = element.attrib.get("s")
            if style_int:
                xfs_style_int = book.xfs_styles[int(style_int)]
                cell.style_string = book.styles.get(str(xfs_style_int))
            parse_cell(cell, book)
            cells.append(cell)
            cell = Cell()
    return [c.value for c in cells]


def parse_cell(cell, book):
    cell.type = parse_cell_type(cell)
    parse_cell_value(cell, book)


def parse_cell_type(cell):
    cell_type = None
    if cell.style_string:
        date_time_flag = (
            re.match(r"^\d+(\.\d+)?$", cell.value)
            and re.match(".*[hsmdyY]", cell.style_string)
            and not re.match(r".*\[.*[dmhys].*\]", cell.style_string)
        )
        if cell.style_string in FORMATS:
            cell_type = FORMATS[cell.style_string]
        elif date_time_flag:
            if float(cell.value) < 1:
                cell_type = "time"
            else:
                cell_type = "date"
        elif re.match(r"^-?\d+(.\d+)?$", cell.value):
            cell_type = "float"
    return cell_type


def parse_cell_value(cell, book):
    if cell.column_type == "s":
        cell.value = book.shared_strings[int(cell.value)]
    elif cell.column_type == "b":
        cell.value = (
            (int(cell.value) == 1 and "TRUE")
            or (int(cell.value) == 0 and "FALSE")
            or cell.value
        )
    elif cell.column_type == "n":
        parse_numeric_cell_value(cell, book)
    # else
    #   no action


def parse_numeric_cell_value(cell, book):
    try:
        if cell.type == "date":  # date/time
            if book.properties["date1904"]:
                start = datetime(1904, 1, 1)
            else:
                start = datetime(1899, 12, 30)
            cell.value = start + timedelta(float(cell.value))
        elif cell.type == "time":  # time
            # round to microseconds
            seconds_in_total = int(
                round((float(cell.value) % 1) * 24 * 60 * 60, 6)
            )
            minutes_in_total = int(seconds_in_total / 60)
            second = int(minutes_in_total % 60)
            hour = int(minutes_in_total / 60)
            # str(t / 60) + ":" + ('0' + str(t % 60))[-2:]
            cell.value = time(
                hour=hour, minute=minutes_in_total % 60, second=second
            )
        elif cell.type == "float" and ("E" in cell.value or "e" in cell.value):
            cell.value = ("%f" % (float(cell.value))).rstrip("0").rstrip(".")
    except (ValueError, OverflowError):
        # invalid date format
        pass


def parse_styles(style_content):
    styles = OrderedDict()
    formats = NUMBER_FMT_MATCHER.findall(style_content)
    for aformat in formats:
        partial = io.BytesIO(aformat)
        for action, element in etree.iterparse(partial, ("end",)):
            if element.tag != "numFmt":
                continue
            numFmtId = element.attrib.get("numFmtId")
            formatCode = element.attrib.get("formatCode")
            formatCode = formatCode.lower().replace("\\", "")
            styles.update({numFmtId: formatCode})
    return styles


def parse_xfs_styles(style_content):
    styles = []
    formats = XFS_FMT_MATCHER.findall(style_content)
    for aformat in formats:
        partial = io.BytesIO(aformat)
        for action, element in etree.iterparse(partial, ("end",)):
            if element.tag != "xf":
                continue
            styles.append(int(element.attrib.get("numFmtId")))
    return styles


def parse_book_properties(book_content):
    properties = {"sheets": []}
    date1904 = DATE_1904_MATCHER.findall(book_content)
    for apr in date1904:
        partial = io.BytesIO(apr)
        for action, element in etree.iterparse(partial, ("end",)):
            if element.tag == "workbookPr":
                value = element.attrib.get("date1904")
                if value:
                    properties["date1904"] = value.lower().strip() == "true"
                else:
                    properties["date1904"] = False

    ns = (
        "http://schemas.openxmlformats.org/"
        + "officeDocument/2006/relationships"
    )
    namespaces = {"r": ns}

    xlsx_header = "<wrapper {0}>".format(
        " ".join('xmlns:{0}="{1}"'.format(k, v) for k, v in namespaces.items())
    ).encode("utf-8")
    xlsx_footer = "</wrapper>".encode("utf-8")
    sheets = SHEET_FMT_MATCHER.findall(book_content)
    for sheet in sheets:
        block = xlsx_header + sheet + xlsx_footer
        partial = io.BytesIO(block)
        for action, element in etree.iterparse(partial, ("end",)):
            if element.tag == "sheet":
                value = element.attrib.get("name")
                properties["sheets"].append(value)
    return properties


def parse_shared_strings(content):
    root = etree.fromstring(content)
    for si in root.iterchildren():
        content = ""
        for text in si.iterchildren():
            if text.text:
                content += text.text
        yield (content)



from io import BytesIO
from datetime import date, time, datetime

import pyexcel_io.service as service
from pyexcel_io.plugin_api import ISheet, IReader, NamedContent
from pyexcel_xlsxr.messy_xlsx import XLSXBookSet


class XLSXSheet(ISheet):
    def __init__(
        self,
        sheet,
        auto_detect_int=True,
        auto_detect_float=True,
        auto_detect_datetime=True,
    ):
        self.xlsx_sheet = sheet
        self.__auto_detect_int = auto_detect_int
        self.__auto_detect_float = auto_detect_float
        self.__auto_detect_datetime = auto_detect_datetime

    def row_iterator(self):
        return self.xlsx_sheet.raw()

    def column_iterator(self, row):
        for cell in row:
            yield self.__convert_cell(cell)

    def __convert_cell(self, cell):
        if cell is None:
            return None
        if isinstance(cell, (datetime, date, time)):
            return cell
        ret = None
        if isinstance(cell, str):
            if self.__auto_detect_int:
                ret = service.detect_int_value(cell)
            if ret is None and self.__auto_detect_float:
                ret = service.detect_float_value(cell)
                shall_we_ignore_the_conversion = (
                    ret in [float("inf"), float("-inf")]
                ) and self.__ignore_infinity
                if shall_we_ignore_the_conversion:
                    ret = None
        if ret is None:
            ret = cell
        return ret


class XLSXBook(IReader):
    def __init__(self, file_alike_object, _, **keywords):
        self.xlsx_book = XLSXBookSet(file_alike_object)
        self._keywords = keywords
        tables = self.xlsx_book.make_tables()
        self.content_array = [
            NamedContent(table.name, table) for table in tables
        ]

    def read_sheet(self, sheet_index):
        """read a sheet at a specified index"""
        table = self.content_array[sheet_index].payload
        sheet = XLSXSheet(table, **self._keywords)
        return sheet

    def close(self):
        self.xlsx_book.close()


class XLSXBookInContent(XLSXBook):
    def __init__(self, file_content, file_type, **keywords):
        file_stream = BytesIO(file_content)
        super().__init__(file_stream, file_type, **keywords)
